/*************************************************************************
 **
 ** mysql C++ wrapper by Howie <caffeine@toodarkpark.org>
 ** translated from the Objective-C mySQL library by Howie <caffeine@toodarkpark.org>
 **
 ** ( loosely based on the mysql-C++ wrapper from Roland Haenel <rh@ginster.net> )
 **
 ************************************************************************/
#include "Database.h"

/****************************************************************/
Database::Database()
{
	connected = false;
	errorMsg="No connection established";
}

/****************************************************************/
Database::~Database()
{
	close();		// Disconnect if connected to database
}

/****************************************************************/
bool Database::isConnected()
{
	return( connected );
}

/****************************************************************/
int Database::init()
{
	return( DB_COMMAND_OK );
}

/****************************************************************/
char *Database::error()
{
	if (connected == false)
	{
		return( ( errorMsg == NULL ? "Not connected" : errorMsg ) );
	}

	return( ( errorMsg == NULL ? mysql_error( &mysql ) : errorMsg ) );
}

/****************************************************************/
bool Database::connect( char *host, char *dbName, char *userName, char *passwd, int port )
{
	if( mysql_connect( &mysql, host, userName, passwd ) == NULL )
	{
		errorMsg="Connect to database failed";
		return( false );
	}

	if( mysql_select_db( &mysql, dbName ) ) 
	{
		errorMsg = mysql_error( &mysql );
		connected = false;
		mysql_close( &mysql );
		return( false );
	}

	connected = true;
	port++;					// cheesy compiler warnings...
	return( true );
}

bool Database::connect( char *host, char *dbName, char *userName, char *passwd )
{
	return( connect( host, dbName, userName, passwd, DB_DEFAULT_PORT ) );
}

bool Database::connect( char *host, char *dbName, char *userName )
{
	return( connect( host, dbName, userName, NULL, DB_DEFAULT_PORT ) );
}

bool Database::connect( char *host, char *dbName)
{
	return( connect( host, dbName, NULL, NULL, DB_DEFAULT_PORT ) );
}

/****************************************************************/
void Database::close()
{
	if( connected == true )
	{
		mysql_close( &mysql );
		connected = false;
	}
}

/****************************************************************/
int Database::reset()
{
	return( DB_COMMAND_OK );
}

/****************************************************************/
DBResult *Database::makeQuery( size_t numBytes, char *query, ...)
{
	va_list ap;
	char *sql=NULL;
	DBResult *res;

	if( isConnected() == false )
	{
		return( NULL );
	}

	if( ( sql = (char *)malloc( numBytes + 3 ) ) == NULL )
	{
		errorMsg="Malloc failed";
		return( NULL );
	}
	else
	{
		va_start( ap, query );
		vsnprintf( sql, 5000, query, ap );
		va_end( ap );

		res = new DBResult( &mysql, sql );
		free( sql );
		return( res );
	}
	
	return( NULL );
}

DBResult *Database::makeQuery( char *query )
{
	if( isConnected() == false )
	{
		return( NULL );
	}

	return( new DBResult( &mysql, query ) );
}

/****************************************************************/
char **Database::makeQuickSelectQuery( char *query )
{
	DBResult *result;
	char **row;
	
	if( !( result = makeQuery( query ) ) )
	{
		return( NULL );
	}
	else
	{
		row = result->getRow();
		delete result;
	}

	return( row );
}

/********************************************************************************/
unsigned int Database::numFields( char * tableName )
{
	MYSQL_RES		*res;
	unsigned int	fieldCount=0;

	if( ( res = mysql_list_fields( &mysql, tableName, NULL ) ) )
	{
		fieldCount = (unsigned long)mysql_num_fields( res );
		mysql_free_result( res );
	}

	return( fieldCount );
}

/********************************************************************************/
char *Database::fieldName( unsigned int fieldNumber, char *tableName )
{
	MYSQL_RES		*res;
	MYSQL_FIELD	field;
	char			*fieldName=NULL;
	
	if( fieldNumber > numFields( tableName ) )
	{
		return( NULL );
	}
	
	res = mysql_list_fields( &mysql, tableName, NULL );
	
	field = mysql_fetch_field_direct( res, fieldNumber );
	fieldName = field.name;
	mysql_free_result( res );
	
	return( fieldName );
}

/********************************************************************************/
unsigned long Database::fieldSize( unsigned int fieldNumber, char *tableName )
{
	MYSQL_RES		*res;
	MYSQL_FIELD	field;
	unsigned long	fieldSize=0L;
	
	if( fieldNumber > numFields( tableName ) )
	{
		return( (unsigned long) 0 );
	}
	
	res = mysql_list_fields( &mysql, tableName, NULL );
	
	field = mysql_fetch_field_direct( res, fieldNumber );
	fieldSize = field.length;
	mysql_free_result( res );
	
	return( fieldSize );
}

unsigned long Database::fieldSize( char *fieldName, char *tableName )
{
	return( fieldSize( fieldByName( fieldName, tableName ), tableName ) );
}

/********************************************************************************/
unsigned int Database::fieldDecimals( unsigned int fieldNumber, char *tableName )
{
	MYSQL_RES		*res;
	MYSQL_FIELD	field;
	unsigned int	fieldDecimals=0;

	if( fieldNumber > numFields( tableName ) )
	{
		return( (unsigned int)NULL );
	}
	
	res = mysql_list_fields( &mysql, tableName, NULL );
	
	field = mysql_fetch_field_direct( res, fieldNumber );
	fieldDecimals = field.decimals;
	mysql_free_result( res );
	
	return( fieldDecimals );
}

unsigned int Database::fieldDecimals( char *fieldName, char *tableName )
{
	return( fieldDecimals( fieldByName( fieldName, tableName ), tableName ) );
}
/********************************************************************************/
unsigned int Database::fieldByName( char *fName, char *tableName )
{
	unsigned int	fieldNum=0;
	unsigned int	fields=0;
	unsigned int	i=0;

	if( ( fields = numFields( tableName ) ) != 65535 )
	{
		for( i=0; i< fields; i++ )
		{
			if( !strcmp( fName, fieldName( i, tableName ) ) )
			{
				fieldNum = i;
			}
		}
		
		return( fieldNum );
	}
	
	return( (unsigned int)NULL );
}
