dnl #########################################################################
dnl The following section which tests for and finds Qt was borrowed from 
dnl the MindsEye aclocal.m4.  It is a very well written macro and I didn't 
dnl want to reinvent the wheel.  I modified it slightly to remove the GL QT
dnl stuff as it is not needed for MySQLTutor.  I also modified some of the
dnl terms and formatting to fit my tastes.
dnl MindsEye is a very cool 3D modeller and its home page is:
dnl http://mindseye.luna.net
dnl #########################################################################

AC_DEFUN(AC_PATH_QT,
[
AC_PROVIDE([$0])
AC_REQUIRE_CPP
AC_REQUIRE([AC_PATH_XTRA])

ac_cv_with_qt_dir=
ac_cv_with_qt_lib=
ac_cv_with_qt_inc=
QT_CFLAGS=
QT_LIBS=

dnl command line options and --help descriptions

AC_ARG_WITH(qt-dir,
    [  --with-qt-dir           where the root of qt is installed (/usr/local/qt)
 ],
    [  ac_cv_with_qt_dir="$withval" ])

AC_ARG_WITH(qt-include,
    [  --with-qt-include       where the qt includes are. (/usr/local/qt/include) ],
    [  ac_cv_with_qt_inc="$withval" ])
    
AC_ARG_WITH(qt-lib,
    [  --with-qt-lib           where the qt library is installed. (/usr/local/qt/lib)],
    [  ac_cv_with_qt_lib="$withval" ])
    
dnl try to set some paths

AC_MSG_CHECKING("for Qt")

for ac_dir in                   \
        ${ac_cv_with_qt_dir}    \
        ${QTDIR}                \
        /usr/local/qt           \
        /usr/share/qt           \
        ;                       \
do
        if test -n "$ac_dir" && test -d "$ac_dir"; then
        ac_cv_with_qt_dir="$ac_dir" 
                break;
        fi
done

if test ! -n "$ac_cv_with_qt_dir"; then
        AC_MSG_RESULT("Nonstandard")
else
        AC_MSG_RESULT("$ac_cv_with_qt_dir")
fi

dnl check for libraries and includes

AC_MSG_CHECKING("for Qt headers")
AC_LANG_SAVE
AC_LANG_CPLUSPLUS

AC_TRY_CPP([#include <qapp.h>],
[
        AC_MSG_RESULT(["Good, headers in your path"])
],
[
AC_MSG_RESULT(["Not in your path, searching..."])
for ac_dir in                   	\
        $ac_cv_with_qt_dir/include      \
        $ac_cv_with_qt_inc              \
    /usr/local/qt/include       	\
    /usr/local/include/qt       	\
    /usr/share/qt/include       	\
    /usr/share/include/qt       	\
    /usr/qt/include             	\
    /usr/include/qt             	\
    /usr/X11R6/include/qt       	\
    /usr/include/X11/qt         	\
    /usr/local/X11R6/include/qt 	\
    /usr/local/X11/include/qt   	\
    /usr/lib/qt/include         	\
    /usr/X11R6/include/qt       	\
    /usr/X11R6/include/X11/qt   	\
    ; 					\
do
   if test -r "$ac_dir/qapp.h"; then
      ac_cv_with_qt_inc=$ac_dir
      break
   fi
done

if test -n "$ac_cv_with_qt_inc" ; then
   AC_MSG_CHECKING(["in $ac_dir"])
   AC_MSG_RESULT(["yes"])
   QT_CFLAGS="-I${ac_cv_with_qt_inc}"
else
   AC_MSG_ERROR(["Can\'t find the Qt headers"])
fi
])
AC_LANG_RESTORE


AC_CHECKING("for Qt library")

SAVE_LIBS="${LIBS}"
    
AC_CHECK_LIB(qt, main,
[
    QT_LIBS="-lqt"
        AC_SUBST(QT_LIBS)
        AC_MSG_RESULT(["Good, library\'s in your path"])
],
[
AC_MSG_RESULT(["Not in your path, searching..."])
for ac_dir in               	\
        $ac_cv_with_qt_dir/lib	\
        $ac_cv_with_qt_lib   	\
    /usr/local/qt/lib       	\
    /usr/local/lib/qt       	\
    /usr/share/qt/lib       	\
    /usr/share/lib/qt       	\
    /usr/qt/lib             	\
    /usr/lib/qt             	\
    /usr/X11R6/lib/qt       	\
    /usr/lib/X11/qt         	\
    /usr/local/X11R6/lib/qt 	\
    /usr/local/X11/lib/qt   	\
    /usr/X11R6/lib          	\
    /usr/lib/X11            	\
    /usr/local/X11R6/lib    	\
    /usr/local/X11/lib      	\
    ; 				\
do
   SAVE2_LIBS="${LIBS}"
   LIBS="${LIBS} -L$ac_dir -lqt  $X_LIBS $X_PRE_LIBS -lX11 $X_EXTRA_LIBS"
   AC_TRY_LINK([],[ main ],
        [
                ac_cv_with_qt_lib=$ac_dir
                QT_LIBS="-L$ac_cv_with_qt_lib -lqt"
                break
        ],
        [])
   LIBS="${SAVE2LIBS}"
done

if test -n "$ac_cv_with_qt_lib" ; then
   AC_MSG_CHECKING(["for -lqt in $ac_dir"])
   AC_MSG_RESULT(["yes"])
else
   AC_MSG_ERROR(["Can\'t find the Qt library"])
fi
],
[$X_LIBS $X_PRE_LIBS -lX11 $X_EXTRA_LIBS])

LIBS="${SAVE_LIBS}"


AC_MSG_RESULT(["+ Qt libraries ${ac_cv_with_qt_lib}, Qt headers ${ac_cv_with_qt_inc}"])


SAVE_PATH="${PATH}"
if test -n $PATH ; then
    PATH="${PATH}:$ac_cv_with_qt_dir/bin"
else
    PATH="$ac_cv_with_qt_dir/bin"
fi

AC_PATH_PROG(MOC, moc, no)
if test "$MOC" = no; then
    AC_MSG_ERROR("You need to install Qt first: can\'t find the meta object compiler")
fi
PATH="${SAVE_PATH}"


AC_SUBST(QT_LIBS)
dnl AC_SUBST(QT_GL_LIBS)
AC_SUBST(QT_CFLAGS)

])


dnl #########################################################################
dnl Check for MySQL headers and libraries
dnl #########################################################################

AC_DEFUN(AC_PATH_MYSQL,
[
AC_PROVIDE([$0])
AC_REQUIRE_CPP
AC_REQUIRE([AC_PATH_XTRA])

ac_cv_with_mysql_dir=
ac_cv_with_mysql_lib=
ac_cv_with_mysql_inc=
MYSQL_CFLAGS=
MYSQL_LIBS=

dnl command line options and --help descriptions

AC_ARG_WITH(mysql-dir,
    [  --with-mysql-dir           where the root of mysql is installed (/usr/local/mysql)
 ],
    [  ac_cv_with_mysql_dir="$withval" ])

AC_ARG_WITH(mysql-include,
    [  --with-mysql-include       where the mysql headers are. (/usr/local/mysql/include) ],
    [  ac_cv_with_mysql_inc="$withval" ])
    
AC_ARG_WITH(mysql-lib,
    [  --with-mysql-lib           where the mysql library is installed. (/usr/local/mysql/lib)],
    [  ac_cv_with_mysql_lib="$withval" ])

dnl find some paths

AC_MSG_CHECKING("for MySQL")

for ac_dir in        		\
        ${ac_cv_with_mysql_dir}	\
        /usr/local/mysql   	\
        /usr/share/mysql       	\
        ;                   	\
do
        if test -n "$ac_dir" && test -d "$ac_dir"; then
        ac_cv_with_mysql_dir="$ac_dir" 
                break;
        fi
done

if test ! -n "$ac_cv_with_mysql_dir"; then
        AC_MSG_RESULT("Nonstandard")
else
        AC_MSG_RESULT("$ac_cv_with_mysql_dir")
fi

dnl check for libraries and includes

AC_MSG_CHECKING("for MySQL headers")

AC_TRY_CPP([#include <mysql.h>],
[
        AC_MSG_RESULT(["Good, headers in your path"])
],
[
AC_MSG_RESULT(["Not in your path, searching..."])
for ac_dir in                  		\
        $ac_cv_with_mysql_dir/include	\
        $ac_cv_with_mysql_inc       	\
    /usr/local/mysql/include       	\
    /usr/local/mysql/include/mysql	\
    /usr/local/include/mysql       	\
    /usr/share/mysql/include       	\
    /usr/share/include/mysql       	\
    /usr/include/mysql             	\
    ; 					\
do
   if test -r "$ac_dir/mysql.h"; then
      ac_cv_with_mysql_inc=$ac_dir
      break
   fi
done

if test -n "$ac_cv_with_mysql_inc" ; then
   AC_MSG_CHECKING(["in $ac_dir"])
   AC_MSG_RESULT(["yes"])
   MYSQL_CFLAGS="-I${ac_cv_with_mysql_inc}"
else
   AC_MSG_ERROR(["Can\'t find the MySQL headers"])
fi
])

AC_CHECKING("for MySQL library path")

SAVE_LIBS="${LIBS}"
    
AC_CHECK_LIB(mysqlclient, main,
[
    MYSQL_LIBS="-lmysqlclient"
        AC_SUBST(MYSQL_LIBS)
        AC_MSG_RESULT(["Good, libraries in your path"])
],
[
AC_MSG_RESULT(["Not in your path, searching..."])
for ac_dir in               		\
        $ac_cv_with_mysql_dir/lib	\
        $ac_cv_with_mysql_lib   	\
    /usr/local/lib/mysql       		\
    /usr/local/mysql/lib       		\
    /usr/local/mysql/lib/mysql		\
    /usr/share/mysql/lib       		\
    /usr/share/lib/mysql       		\
    /usr/lib/mysql             		\
    ; 					\
do
   SAVE2_LIBS="${LIBS}"
   LIBS="${LIBS} -L$ac_dir -lmysqlclient -lm"
   AC_TRY_LINK([],[ main ],
        [
                ac_cv_with_mysql_lib=$ac_dir
                MYSQL_LIBS="-L$ac_cv_with_mysql_lib -lmysqlclient -lm"
                break
        ],
        [ ])
   LIBS="${SAVE2LIBS}"
done

if test -n "$ac_cv_with_mysql_lib" ; then
   AC_MSG_CHECKING(["for -lmysqlclient in $ac_dir"])
   AC_MSG_RESULT(["yes"])
else
   AC_MSG_ERROR(["Can\'t find the MySQL library"])
fi
], [-lm])

LIBS="${SAVE_LIBS}"


AC_MSG_RESULT(["+ MySQL libraries ${ac_cv_with_mysql_lib}, MySQL headers ${ac_cv_with_mysql_inc}"])

AC_SUBST(MYSQL_LIBS)
AC_SUBST(MYSQL_CFLAGS)

])
