/*************************************************************************
 **
 ** mysql C++ wrapper by Howie <caffeine@toodarkpark.org>
 ** translated from the Objective-C mySQL library by Howie <caffeine@toodarkpark.org>
 **
 ** ( loosely based on the mysql-C++ wrapper from Roland Haenel <rh@ginster.net> )
 **
 ************************************************************************/


#ifndef __CPP_SQL_DATABASE_H
#define __CPP_SQL_DATABASE_H

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <stdarg.h>

// database specific
#include "mysql.h"

#define DB_DEFAULT_PORT		(int)3333

#define DB_CONNECTION_NONE	0
#define DB_CONNECTION_OK		1
#define DB_CONNECTION_BAD	2

#define DB_COMMAND_OK		0	// OK - command executed
#define DB_EMPTY_QUERY		1	// Query didnt return tuples
#define DB_TUPLES_OK		2	// Query returned tuples
#define DB_ERROR			5
#define DB_BAD_RESPONSE		6
#define DB_UNEXPECTED		7	// This shouldnt ever happen

typedef enum
{
  FIELDTYPE_CHAR=0,
  FIELDTYPE_INT,
  FIELDTYPE_LONG,
  FIELDTYPE_TEXT,
  FIELDTYPE_DATE,
  FIELDTYPE_TIME,
  FIELDTYPE_DATETIME,
  FIELDTYPE_DECIMAL,
  FIELDTYPE_DOUBLE,
  FIELDTYPE_FLOAT,
  FIELDTYPE_MONEY,
} sqlFieldType;

class DBResult;	// well declare that later

class DBAdmin; // this too

class Database 
{
protected:
  MYSQL mysql;
  bool connected;
  char *errorMsg;
  
public:
  Database();
  ~Database();
  
  int init();
  bool isConnected();
  char *error();
  
  // Connecting to the database
  bool connect( char *host, char *dbName, char *userName, char *passwd, int port );
  bool connect( char *host, char *dbName, char *userName, char *passwd );
  bool connect( char *host, char *dbName, char *userName );
  bool connect( char *host, char *dbName );
  
  // making queries
  DBResult *makeQuery( char *query );
  DBResult *makeQuery( size_t numBytes, char *query, ... );
  DBResult *makeQuery( DBResult *result, char *query );
  DBResult *makeQuery( DBResult *result, size_t numBytes, char *query, ... );
  
  // making quick select queries returning 1 row of data
  char **makeQuickSelectQuery( char *query );
  char **makeQuickSelectQuery( size_t numBytes, char *query, ... );
  
  // admin-type functions
  // bool dropTable( char *tableName );    DBAdmin.cc
  // bool dropDatabase( char *dbName );    DBAdmin.cc
  bool createDB(char *dbName);
  bool dropDB(char *dbName);
  bool reload();
  bool shutdown();

  
  // database/table information
  DBResult *listFields( char *fromTable );
  DBResult *listTables();
  DBResult *listDatabases();
  
  // accessing fields
  unsigned int numFields( char *inTable );
  
  char *fieldName( unsigned int fieldNumber, char *tableName );
  unsigned int fieldByName( char *fieldName, char *tableName );
  
  sqlFieldType fieldType( unsigned int fieldNumber, char *tableName );
  sqlFieldType fieldType( char *fieldName, char *tableName );
  
  unsigned long fieldSize( unsigned int fieldNumber, char *tableName );
  unsigned long fieldSize( char *fieldName, char *tableName );
  
  unsigned int fieldDecimals( unsigned int fieldNumber, char *tableName );
  unsigned int fieldDecimals( char *fieldName, char *tableName );
  
  void close();
  int reset();
  
  // transactions
  bool transactionBegin();
  bool transactionCommit();
  bool transactionRollback();
};



class DBResult
{
protected:
  MYSQL *mysql;
  bool resError;
  char *errorMsg;
  MYSQL_RES *result;
  
public:
  DBResult();
  DBResult( MYSQL *db_mysql, char *query );
  ~DBResult();
  
  int status();									// Return query status
  char *error();
  
  void makeQuery( char *query );
  void makeQuery( size_t numBytes, char *query, ... );
  
  unsigned int numFields();
  unsigned long numRows();
  
  // accessing fields
  char *fieldName( unsigned int fieldNumber );
  unsigned int fieldByName( char *fieldName );
  
  sqlFieldType fieldType( unsigned int fieldNumber );
  sqlFieldType fieldType( char *fieldName );
  
  unsigned long fieldSize( unsigned int fieldNumber );
  unsigned long fieldSize( char *fieldName );
  
  unsigned int fieldDecimals( unsigned int fieldNumber );
  unsigned int fieldDecimals( char *fieldName );
  
  // retrieving data
  void goToRow( int rowNumber );
  char **getRow();
  char **getRow( int rowNumber );
  
  // misc functions
  unsigned long lastID();
};

class DBAdmin
{
protected:
  MYSQL mysql;
  char  *errorMsg;

public:
  DBAdmin();
  ~DBAdmin();

  bool createDatabase(const char *dbName);
  bool dropDatabase(char *dbName);
  bool reloadDatabase();
  bool shutdownDatabase();
};


#endif
