/*************************************************************************
 **
 ** mysql C++ wrapper by Howie <caffeine@toodarkpark.org>
 ** translated from the Objective-C mySQL library by Howie <caffeine@toodarkpark.org>
 **
 ** ( loosely based on the mysql-C++ wrapper from Roland Haenel <rh@ginster.net> )
 **
 ************************************************************************/

#include "Database.h"
#include <iostream.h>

/********************************************************************************/
DBResult::DBResult()
{
	result     = NULL;
	resError  = false;
}

/********************************************************************************/
DBResult::DBResult( MYSQL *db_mysql, char *query )
{
	result     = NULL;
	resError  = false;

	// setup inherited mysql...	
	mysql = db_mysql;
	
	makeQuery( query );
}

/********************************************************************************/
DBResult::~DBResult()
{
	if( result != NULL )							// Free memory resources
		mysql_free_result( result );
}

/********************************************************************************/
void DBResult::makeQuery( char *query )
{
	if( result != NULL )
	{
		mysql_free_result( result );
		result = NULL;
	}
	
	if( mysql_query( mysql, query ) == -1 )			// query failed, so return 'nil'.
	{
		errorMsg = mysql_error( mysql );
	}
	else
	{
		if( ( result = mysql_store_result( mysql ) ) == NULL )
		{
			resError = true;
		
			if( mysql_error( mysql  ) == NULL )
			{
				errorMsg = "No result given";
			}
			else
			{
				errorMsg = mysql_error( mysql );
			}
		}
	}
}

/********************************************************************************/
int DBResult::status()
{
	if( resError )
	{
		return DB_ERROR;
	}

	if( result == NULL ) 
	{
		return DB_COMMAND_OK;
	}

	return DB_TUPLES_OK;
}

/********************************************************************************/
unsigned long DBResult::numRows()
{
	if( result == NULL )
	{
		return( mysql_affected_rows( mysql ) );
	}
	else
	{
		return( (unsigned long)mysql_num_rows( result ) );
	}
}

/********************************************************************************/
unsigned int DBResult::numFields()
{
	return( ( result == NULL ? 0 : mysql_num_fields( result ) ) );
}

/********************************************************************************/
void DBResult::goToRow( int row )
{
	if( result != NULL )
	{
		mysql_data_seek( result, row );
	}
}

/********************************************************************************/
char **DBResult::getRow()
{
	if( result != NULL )
	{
		return( mysql_fetch_row( result ) );
	}
	else
	{
		//printf( "result == NULL\n" );
		return( NULL );
	}
}

/********************************************************************************/
char **DBResult::getRow( int row )
{
	goToRow( row );
	return( getRow() );
}

/********************************************************************************/
char *DBResult::fieldName( unsigned int fieldNumber )
{
	MYSQL_FIELD field;
	
	if( result == NULL )
	{
		resError = true;
		return( NULL );
	}
	
	field = mysql_fetch_field_direct( result, fieldNumber );
	
	return( field.name );
}

/********************************************************************************/
unsigned long DBResult::fieldSize( unsigned int fieldNumber )
{
	MYSQL_FIELD field;
	
	if( result == NULL )
	{
		return( (unsigned long) NULL );
	}
	
	if( fieldNumber > numFields() )
	{
		return( (unsigned long) NULL );
	}
	
	field = mysql_fetch_field_direct( result, fieldNumber );
	
	if( field.name == NULL ) 
	{
		return( (unsigned long) NULL );
	}

	else
	{
		return( (unsigned long)field.length );
	}
}

/****************************************************************/
sqlFieldType DBResult::fieldType( unsigned int fieldNumber )
{
	MYSQL_FIELD field;
	
	if( result != NULL )
	{
		if( fieldNumber > numFields() )
		{
			return( (sqlFieldType)-1 );
		}
	
		field = mysql_fetch_field_direct( result, fieldNumber );
	
		if( field.name == NULL ) 
		{
			return( (sqlFieldType)-1 );
		}

		else
		{
			return( (sqlFieldType)field.type );
		}
	}

	return( (sqlFieldType)-1 );
}

/****************************************************************/
unsigned int DBResult::fieldDecimals( unsigned int fieldNumber )
{
	MYSQL_FIELD field;
	
	if( result == NULL )
	{
		return( (unsigned int)NULL );
	}
	
	if( fieldNumber > numFields() )
	{
		return( (unsigned int)NULL );
	}
	
	field = mysql_fetch_field_direct( result, fieldNumber );
	
	if( field.name == NULL ) 
	{
		return( (unsigned int)NULL );
	}

	else
	{
		return( field.decimals );
	}
}

/****************************************************************/
unsigned int DBResult::fieldByName( char *fieldName )
{
	MYSQL_FIELD	field;
	int			i=0;
	int			fields = 0;
	int			fieldnum = -1;
	
	fields = numFields();
	
	for( i=0; i<fields; i++ )
	{
		field = mysql_fetch_field_direct( result, i );
	
		if( field.name != NULL )
		{
			if( !strcmp( field.name, fieldName ) )
			{
				fieldnum = i;
			}
		}
	}
	
	return( i );
}
/********************************************************************************/
unsigned long DBResult::lastID()
{
	return( mysql_insert_id( mysql ) );
}
/****************************************************************/
char *DBResult::error()
{
	return( ( errorMsg == NULL ? mysql_error( mysql ) : errorMsg ) );
}

