/*!
  \class ConfigDialog configdlg.h
  \brief This class is the configuration interface dialog.

  This class is the configuration dialog.  It inherits QTabDialog and contains
  several other widgets as "pages" to the dialog.  These other widgets will contain
  items for data entry, selection mechanisms, etc. etc.
*/

#include "configdlg.h"

#include "dbconf.h"
#include "startupconf.h"

/*!
  This is the constructor.  Currently it gets passed a QWidget pointer and 
  a const char* pointer as "parent" and "name" respectively.
  
  This constructor creates each of the widget "pages" to add to the tab dialog.
*/

ConfigDialog::ConfigDialog( QWidget *parent, const char *name )
  : QTabDialog( parent, name )
{
  setCaption( "MySQLTutor Options" );
  setApplyButton();
  setCancelButton();

  dbConf = new DBConfigWidget( this, "dbconf" );
  suConf = new SUConfigWidget( this, "suconf" );

  // database configuration
  hostEdP = dbConf->getHostPtr();
  userEdP = dbConf->getUserPtr();
  passEdP = dbConf->getPassPtr();
  dbEdP = dbConf->getDBPtr();

  // statup configuration
  conYesP = suConf->getconYesPtr();
  lessYesP = suConf->getlesYesPtr();
  geoYesP = suConf->getgeoYesPtr();
  splitYesP = suConf->getsplitYesPtr();
  dirEditP = suConf->getDirEditPtr();

  addTab( dbConf, "Database" );
  addTab( suConf, "Start Up" );

  resize( 400, 400 );
}

/*!
  This is the destructor, it currently does nothing.
*/

ConfigDialog::~ConfigDialog(){}

/*!
  \fn QLineEdit *ConfigDialog::getHostPtr()
  This member returns a pointer to the host edit box on the database configuration
  widget.
*/

/*!
  \fn QLineEdit *ConfigDialog::getUserPtr()
  This member returns a pointer to the user edit box on the database configuration
  widget.
*/

/*!
  \fn QLineEdit *ConfigDialog::getPassPtr()
  This member returns a pointer to the password edit box on the database configuration
  widget.
*/

/*!
  \fn QLineEdit *ConfigDialog::getDBPtr()
  This member returns a pointer to the database edit box on the database configuration
  widget.
*/

/*!
  \fn QRadioButton *ConfigDialog::getConYesPtr()
  This member returns a pointer to the setup configuration's autoconnect "Yes" button.
*/

/*!
  \fn QRadioButton *ConfigDialog::getLessYesPtr() 
  This member returns a pointer to the setup configurations start on last lesson "Yes" button.
*/

/*!
  \fn QRadioButton *ConfigDialog::getGeoYesPtr()
  This member returns a pointer to the setup configurations keep geometry "Yes" button.
*/

/*!
  \fn QRadioButton *ConfigDialog::getSplitYesPtr()
  This member returns a pointer to the setup configurations keep splitter "Yes" button.
*/

/*!
  \fn QLineEdit *ConfigDialog::getDirEdPtr()
  This member returns a pointer to the setup configurations lesson directory edit.
*/
