/*!
  \class DBConfigWidget dbconf.h
  \brief This is the interface widget for database configuration information.

  This widget serves as a "page" in the ConfigDialog widget.  This particular "page"
  allows the user to enter database configuration information such as host, username,
  password, and database.
*/

#include "dbconf.h"

#include <qlabel.h>
#include <qlined.h>
#include <qlayout.h>

/*!
  This is the constructor.  It contains a QGridLayout for geometry management.
  This is the easiest way of doing the geometry management for these types of 
  widgets.  I must currently use the resize() function to set the widget to
  300x300 until I can figure out some other method.
*/

DBConfigWidget::DBConfigWidget( QWidget *parent, const char *name )
  : QWidget( parent, name )
{
  resize( 300, 300 );


  QGridLayout layOut( this, 6, 4 );

  hostEdit = new QLineEdit( this, "hosted" );
  layOut.addWidget( hostEdit, 1, 3 );

  QLabel *hostLabel = new QLabel( "Host:", this );
  hostLabel->setMinimumSize( hostLabel->sizeHint() );
  layOut.addWidget( hostLabel, 1, 1 );

  userEdit = new QLineEdit( this, "usered" );
  layOut.addWidget( userEdit, 2, 3 );

  QLabel *userLabel = new QLabel( "Username:", this );
  userLabel->setMinimumSize( hostLabel->sizeHint() );
  layOut.addWidget( userLabel, 2, 1 );

  passEdit = new QLineEdit( this, "passed" );
  passEdit->setEchoMode( QLineEdit::Password );
  layOut.addWidget( passEdit, 3, 3 );

  QLabel *passLabel = new QLabel( "Password:", this );
  passLabel->setMinimumSize( passLabel->sizeHint() );
  layOut.addWidget( passLabel, 3, 1 );

  dbEdit = new QLineEdit( this, "dbed" );
  layOut.addWidget( dbEdit, 4, 3 );

  QLabel *dbLabel = new QLabel( "Database:", this );
  dbLabel->setMinimumSize( dbLabel->sizeHint() );
  layOut.addWidget( dbLabel, 4, 1 );


  layOut.activate();
}

/*!
  This is the destructor.  It currently does nothing.
*/

DBConfigWidget::~DBConfigWidget(){}

/*!
  \fn QLineEdit *DBConfigWidget::getHostPtr()
  This member returns a pointer to the host editor.
*/

/*!
  \fn QLineEdit *DBConfigWidget::getUserPtr()
  This member returns a pointer to the username editor.
*/

/*!
  \fn QLineEdit *DBConfigWidget::getPassPtr()
  This member returns a pointer to the password editor.
*/

/*!
  \fn QLineEdit *DBConfigWidget::getDBPtr()
  This member returns a pointer to the database name editor.
*/
