/*!
  \class SQLTutorAppWindow sqltutor.h
  \brief The SQLTutorAppWindow class is the main window layout for MySQLTutor.

  This is the main window of MySQLTutor.  I initializes all the child widgets and
  any child classes such as the database object.
*/

#include "sqltutor.h"
#include "mainwidget.h"
#include "sqltable.h"

#include "Database.h"

#include <qpixmap.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qstatusbar.h>
#include <qapplication.h>
#include <qwhatsthis.h>
#include <qkeycode.h>
#include <qaccel.h>
#include <qmsgbox.h>
#include <qmultilinedit.h>

// temp
#include <iostream.h>
#include <string>

#include "pixmaps.h"

const char * newLessonText = "Click this button to create a new lesson.\n\n"
                             "This feature is not yet implemented";

const char * editLessonText = "Click this button to open a lesson for editing.\n\n"
                              "This feature is not yet implemented";

const char * moveBackText = "Click this button to move back one lesson.";

const char * moveTopText = "Click this button to move to the top (first) lesson.";

const char * moveForwardText = "Click this button to move forward one lesson.";

const char * connectText = "Click this button to connect or disconnect from the MySQL Database.\n\n"
                                 "When the circle is green, there is a connection.\n"
                                 "When the circle is red, there is no connection.";

const char * dbConfigText = "Click this button to configure the database options.";

const char * runText = "Click this button to run the query in the example editor.\n\n"
                       "The results of the query will be shown at the bottom in the results table.";

const char * clearText = "Click this button to clear the example editor.\n\n"
                         "This will clear everything in the example editor and leave it blank.\n"
                         "Any changes you have made will be lost.";

const char * recycleText = "Click this button to reload the example editor with the original example.\n\n"
                           "This will bring up the original example for the current lesson.\n"
                           "Any changes you have made will be lost.";

const char * helpText = "Click this button to bring up MySQLTutor's online help.";


/*!

  This is the constructor, it takes an argument of a ConfigManager.
  I pass it a value pointer to the ConfigManager so that I can retrieve
  the configuration values for actions such as database connections, etc. etc.

*/

SQLTutorAppWindow::SQLTutorAppWindow( ConfigManager * configParser )
  : QMainWindow( 0, "mysqltutor main window" ), fParser(configParser)
{

  // Declare the pixmaps for the toolbar buttons
  QPixmap newIcon, editIcon, backIcon, topIcon, fwdIcon, 
    configIcon, runIcon, clearIcon, recycleIcon, helpIcon;


  centerWidget = new TutorMainWidget( this, "main widget", fParser );
  centerWidget->setFocus();
  setCentralWidget( centerWidget );
  QObject::connect( centerWidget, SIGNAL(lessonChanged()), this, SLOT(setStatusBar()) );

  // Declare the toolbar
  controls = new QToolBar( "MySQLTutor Controls", this, Top, FALSE, "control bar" );

  // Declare the toolbuttons
  newIcon = QPixmap( new_xpm );
  QToolButton * newLesson = new QToolButton( newIcon, "New Lesson", 0,
					     this, SLOT(newlesson()),
					     controls, "new lesson" );

  editIcon = QPixmap( open_xpm );
  QToolButton * editLesson = new QToolButton( editIcon, "Edit Lesson", 0,
					      this, SLOT(editlesson()),
					      controls, "edit lesson" );
  
  controls->addSeparator();
  controls->addSeparator();
  controls->addSeparator();

  backIcon = QPixmap( back_xpm );
  QToolButton * backLesson = new QToolButton( backIcon, "Previous Lesson", 0,
					      centerWidget, SLOT(backlesson()),
					      controls, "previous lesson" );

  topIcon = QPixmap( top_xpm );
  QToolButton * topLesson = new QToolButton( topIcon, "Top Lesson", 0,
					     centerWidget, SLOT(toplesson()),
					     controls, "top lesson" );

  fwdIcon = QPixmap( fwd_xpm );
  QToolButton * fwdLesson = new QToolButton( fwdIcon, "Next Lesson", 0,
					     centerWidget, SLOT(fwdlesson()),
					     controls, "next lesson" );

  controls->addSeparator();
  controls->addSeparator();
  controls->addSeparator();

  connectIcon = QPixmap( connected_xpm );
  dconnectIcon = QPixmap( disconnected_xpm );
  connectButton = new QToolButton( dconnectIcon, "Connect/Disconnect", 0,
				   this, SLOT(dbconnect()),
				   controls, "connect button" );
  
  configIcon = QPixmap( dbopts_xpm );
  QToolButton * configButton = new QToolButton( configIcon, "Database Configuration", 0,
						this, SLOT(dbconfig()),
						controls, "config button" );

  controls->addSeparator();
  controls->addSeparator();
  controls->addSeparator();

  runIcon = QPixmap( run_xpm );
  QToolButton * runButton = new QToolButton( runIcon, "Run Example", 0,
					     this, SLOT(runexample()),
					     controls, "run button" );

  controls->addSeparator();
  controls->addSeparator();
  controls->addSeparator();

  clearIcon = QPixmap( undo_xpm );
  QToolButton * clearButton = new QToolButton( clearIcon, "Clear Example", 0,
					       this, SLOT(clearexample()),
					       controls, "clear button" );

  recycleIcon = QPixmap( recycle_xpm );
  QToolButton * recycleButton = new QToolButton( recycleIcon, "Reload Examle", 0,
						 centerWidget, SLOT(reloadexample()),
						 controls, "recycle button" );

  controls->addSeparator();
  controls->addSeparator();
  controls->addSeparator();

  helpIcon = QPixmap( help_xpm );
  QToolButton * helpButton = new QToolButton( helpIcon, "Online Help", 0,
					      this, SLOT(showhelp()),
					      controls, "help button" );

  controls->addSeparator();
  controls->addSeparator();
  controls->addSeparator();

  (void)QWhatsThis::whatsThisButton( controls );
  QWhatsThis::add( newLesson, newIcon, "New Lesson", newLessonText,  FALSE );
  QWhatsThis::add( editLesson, editIcon, "Edit Lesson", editLessonText, FALSE );
  QWhatsThis::add( backLesson, backIcon, "Back One Lesson", moveBackText, FALSE );
  QWhatsThis::add( topLesson, topIcon, "First Lesson", moveTopText, FALSE );
  QWhatsThis::add( fwdLesson, fwdIcon, "Forward One Lesson", moveForwardText, FALSE );
  QWhatsThis::add( connectButton, connectIcon, "Connect to database", connectText, FALSE );
  QWhatsThis::add( configButton, configIcon, "Database Configuration", dbConfigText, FALSE );
  QWhatsThis::add( runButton, runIcon, "Run Example Query", runText, FALSE );
  QWhatsThis::add( clearButton, clearIcon, "Clear Example Editor", clearText, FALSE );
  QWhatsThis::add( recycleButton, recycleIcon, "Reload Example Editor", recycleText, FALSE );
  QWhatsThis::add( helpButton, helpIcon, "Online Help", helpText, FALSE );

  // menu bar and associated items
  QPopupMenu * file = new QPopupMenu();
  file->insertItem("New Lesson", -1, -1);
  file->insertItem("Open Lesson for editing", -1, -1);
  file->insertItem( "Exit",  qApp, SLOT(quit()), CTRL+Key_Q  );

  menuBar()->insertItem( "&File", file );

  QPopupMenu * lesson = new QPopupMenu();
  // lesson->insertItem("Lesson List", lessonlist, 100, -1 );
  lesson->insertItem("Next Lesson", centerWidget, SLOT(fwdlesson()), CTRL+Key_N );
  lesson->insertItem("Previous Lesson", centerWidget, SLOT(backlesson()), CTRL+Key_P );
  lesson->insertItem("Top", centerWidget, SLOT(toplesson()), CTRL+Key_H );
  
  menuBar()->insertItem( "&List", lesson );

  QPopupMenu * action = new QPopupMenu();
  action->insertItem("(Dis)Connect to database", this, SLOT(dbconnect()));
  action->insertSeparator();
  action->insertItem("Clear Example", this, SLOT(clearexample()));
  action->insertItem("Reload Example", centerWidget, SLOT(reloadexample()));
  action->insertItem("Clear Results", this, SLOT(clearresults()));
  action->insertSeparator();
  action->insertItem("Run Example", this, SLOT(runexample()));
  
  menuBar()->insertItem( "&Action", action );

  QPopupMenu * options = new QPopupMenu();
  options->insertItem("Edit Defaults", this, SLOT(dbconfig()));
  options->insertItem("Startup options", this, SLOT(dbconfig()));
  options->insertItem("Database configuration", this, SLOT(dbconfig()));

  menuBar()->insertItem( "&Options", options );
  
  appearance = new QPopupMenu();
  menuBar()->insertItem( "&Controls", appearance );

  mb = appearance->insertItem( "Menu bar", this, SLOT(toggleMenuBar()), CTRL+Key_M);
  // Now an accelerator for when the menubar is invisible!
  QAccel* a = new QAccel(this);
  a->connectItem( a->insertItem( CTRL+Key_M ), this, SLOT(toggleMenuBar()) );
  
  tb = appearance->insertItem( "Tool bar", this, SLOT(toggleToolBar()), CTRL+Key_T);
  sb = appearance->insertItem( "Status bar", this, SLOT(toggleStatusBar()), CTRL+Key_B);
  appearance->setCheckable( TRUE );
  appearance->setItemChecked( mb, TRUE );
  appearance->setItemChecked( tb, TRUE );
  appearance->setItemChecked( sb, TRUE );

  QPopupMenu * help = new QPopupMenu();
  help->insertItem("About", this, SLOT(showabout()));
  help->insertItem("Help", this, SLOT(showhelp()));
  help->insertSeparator();
  help->insertItem("Register", -1, -1);

  menuBar()->insertSeparator();
  menuBar()->insertItem( "&Help", help );
  
  if( fParser->getLessonTrack() == true )
    centerWidget->loadLessonText( fParser->getLastLesson() );
  else
    centerWidget->loadLessonText( 0 );

  statusBar()->message( centerWidget->getLessonLabel() );
  setUsesBigPixmaps( fParser->getIconStatus() );

  //  fParser->setDBInfo();

  sqlDb = new Database();

  if ( fParser->getAutoConnect() )
    dbconnect();

  // setConfigValues();
}

/*!
  This is the destructor.  It currently does nothing.
*/

SQLTutorAppWindow::~SQLTutorAppWindow() {}

/*!
  This is the newlesson slot.  It has not yet been implemented.  It will be used
  once the Lesson Editor is completed.
*/

void SQLTutorAppWindow::newlesson(){}

/*!
  This is the editlesson slot.  It has not yet been implemented.  It will be used
  once the Lesson Editor is completed.
*/

void SQLTutorAppWindow::editlesson(){}

/*!
  This is the dbconnect slot.  This slot initiates a database connection using 
  a Database object created in the constructor.  If this slot is called when a connection
  is active, it will disconnect from the database.
*/

void SQLTutorAppWindow::dbconnect()
{
  if( sqlDb->isConnected() == true )
    {
      sqlDb->close();
      connectButton->setPixmap( dconnectIcon );
      return;
    }
  else
    {
      if (sqlDb->connect( const_cast<char*>(fParser->getHost()), const_cast<char*>(fParser->getDBase()), const_cast<char*>(fParser->getUser()), const_cast<char*>(fParser->getPass()) ) == true)
	connectButton->setPixmap( connectIcon );
      else
	{
	  switch( QMessageBox::warning( this, "MySQLTutor 1.0",
					"Connection to the MySQL database failed.\n"
					"No examples can be run without a MySQL connection\n"
					"Do you want to configure your connection now?",
					"C&ancel", "&Configure", 
					0,
					1 ) )
	    {
	    case 0:
	      break;
	    case 1:
	      // dbConfig();
	      break;
	    }
	}
    }
}

/*!
  This is the dbconfig slot.  It will show the configuration dialog of the
  ConfigManager class.
*/

void SQLTutorAppWindow::dbconfig(){ fParser->showDialog(); }

/*!
  This is the runexample slot.  This slot grabs the text from the example editor
  and sends it to the Database object.  I would like to add some parsing to allow
  multiple queries separated by a ';' character, as in the real SQL command line.
*/

void SQLTutorAppWindow::runexample(){
  string queryString;
  int rows, fields;
  QMultiLineEdit * tempEd = centerWidget->getExampleHandle();
  SQLTable * tempTable = centerWidget->getTableHandle();

  if( sqlDb->isConnected() )
    {
      queryString = tempEd->text();
      
      dbres = sqlDb->makeQuery(const_cast<char*>(queryString.c_str()));
      
      rows = dbres->numRows();
      fields = dbres->numFields();

      tempTable->clearTable();
      if( rows > 0 )
   	{
   	  tempTable->setRows( rows + 1 );
   	  tempTable->setCols( fields );
   	}
         else
	   {
	     tempTable->setRows( 1 );
	     tempTable->setCols( 1 );
	   }
      

      if( rows == 1 && fields == 0 )
	{
	  tempTable->setCell( 0, 0, "Query successfull" );
	  tempTable->setCell( 1, 0, "No results shown" );
	  return;
	}

      if( rows == 0 && fields == 0 )
	{
	  tempTable->setCell( 0, 0, "Query resulted in" );
	  tempTable->setCell( 1, 0, "0 Rows and 0 Fields" );
	  return;
	}

      if( rows < 0 )
	{
	  tempTable->setCell( 0, 0, sqlDb->error() );
	}
      else
	{
	  for( int f = 0; f < fields; f++ )
	    {
	      tempTable->setCell( 0, f, dbres->fieldName( f ) );
	    }

	  for( int r = 0; r < rows; r++ )
	    {
	       for( int c = 0; c < fields; c++)
		 {
		   tempTable->setCell( r + 1, c, dbres->getRow(r)[c]); 
		 }
	    }
	}
      
    }
  else
    {
      tempTable->clearTable();
      tempTable->setRows( 1 );
      tempTable->setCols( 1 );
      tempTable->setCell( 0, 0, "Not Connected to MySQL" );
    }
}

/*!
  This is the clearexample slot.  This slot merely sets the example editor's text
  to "", or blank.
*/

void SQLTutorAppWindow::clearexample()
{
  QMultiLineEdit * tempedit = centerWidget->getExampleHandle();
  tempedit->setText("");
}

/*!
  This is the clearresults slot.  This slot merely clears the SQLTable.
*/

void SQLTutorAppWindow::clearresults(){
  SQLTable * temp = centerWidget->getTableHandle();
  temp->clearTable();
}

/*!
  This is the showhelp slot.  This slot will show the online help, once it becomes
  available.  Currently it does nothing.
*/

void SQLTutorAppWindow::showhelp(){ }

/*!
  This is the showabout slot.  This slot brings up a message box with a brief description
  of MySQLTutor and the authors name and email address.
*/

void SQLTutorAppWindow::showabout()
{
  QMessageBox::information( this, "MySQLTutor",
			    "MySQLTutor Copyright 1998\n"
			    "       Scott Barron      \n"
			    "  <sbarron@vvm.com>        "
			    );
}

/*!
  This is the toggleMenuBar slot.  It is used to show and hide the applications
  menu bar.  If the menu bar is visible when this slot is called, it will be hidden.
  If the menu bar is hidden when this slot is called, it will be shown.
*/

void SQLTutorAppWindow::toggleMenuBar()
{
    if ( menuBar()->isVisible() ) {
	menuBar()->hide();
	appearance->setItemChecked( mb, FALSE );
    } else {
	menuBar()->show();
	appearance->setItemChecked( mb, TRUE );
    }
}

/*!
  This is the tobbleToolBar slot.  It is used to show and hide the applications
  tool bar.  If the tool bar is visible when this slot is called, it will be hidden.
  If the tool bar is hidden when this slot is called, it will be shown.
*/

void SQLTutorAppWindow::toggleToolBar()
{
    if ( controls->isVisible() ) {
	controls->hide();
	appearance->setItemChecked( tb, FALSE );
    } else {
	controls->show();
	appearance->setItemChecked( tb, TRUE );
    }
}

/*!
  This is the toggleStatusBar slot.  It is used to show and hide the applications
  status bar.  If the status bar is visible whenthe slot is called, it will be hidden.
  If the status bar is hidden when the slot is called, it will be shown.
*/

void SQLTutorAppWindow::toggleStatusBar()
{
    if ( statusBar()->isVisible() ) {
	statusBar()->hide();
	appearance->setItemChecked( sb, FALSE );
    } else {
	statusBar()->show();
	appearance->setItemChecked( sb, TRUE );
    }
}

/*!
  This is the setStatusBar function.  This function sets the text content of the status
  bar to the name of the current SQL lesson.
*/

void SQLTutorAppWindow::setStatusBar()
{
  statusBar()->message(centerWidget->getLessonLabel());
}
