/*!
  \class SUConfigWidget startupconf.h
  \brief This widget is the page of the configuration dialog handling startup options.

  This widget is the page of the configuration dialog which handles the startup options.
  On this page are radio buttons to select Auto Connect, Start on Last Lesson, Keep
  Geometry, and Keep Splitter Positions as well as a QLineEdit containing the directory
  in which the lessons are stored.
*/

#include "startupconf.h"

#include <qradiobutton.h>
#include <qlined.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qbuttongroup.h>

/*!
  This is the constructor.  It passes the QWidget parent and char name on to the 
  QWidget constructor.  This is ugly, I know.  But it works for now.
*/

SUConfigWidget::SUConfigWidget( QWidget *parent, const char *name )
  : QWidget( parent, name )
{
  resize( 300, 300 );

 
  QGridLayout layOut( this, 9, 6 );

  // Yes and No labels
  QLabel *yesLabel = new QLabel( "Yes", this, "yes label" );
  yesLabel->setMinimumSize( yesLabel->sizeHint() );
  layOut.addWidget( yesLabel, 1, 3 );

  QLabel *noLabel = new QLabel( "No", this, "no label" );
  noLabel->setMinimumSize( noLabel->sizeHint() );
  layOut.addWidget( noLabel, 1, 5 );

  // Auto connect stuff
  QLabel *connectLabel = new QLabel( "Auto Connect", this, "autocon label" );
  connectLabel->setMinimumSize( connectLabel->sizeHint() );
  layOut.addWidget( connectLabel, 2, 1 );

  QButtonGroup *conGrp = new QButtonGroup( this, "connect group" );
  conGrp->setLineWidth( 0 );

  connectYes = new QRadioButton( this, "connect yes" );
  connectYes->setStyle( WindowsStyle );
  layOut.addWidget( connectYes, 2, 3 );
  
  connectNo = new QRadioButton( this, "connect no" );
  connectNo->setStyle( WindowsStyle );
  layOut.addWidget( connectNo, 2, 5 );

  conGrp->insert( connectYes );
  conGrp->insert( connectNo );
  connectNo->setChecked( true );

  // Start on last lesson stuff
  QLabel *startLess = new QLabel( "Start on Last Lesson", this, "start on last label" );
  startLess->setMinimumSize( startLess->sizeHint() );
  layOut.addWidget( startLess, 3, 1 );

  QButtonGroup *lesGrp = new QButtonGroup( this, "lesson group" );
  lesGrp->setLineWidth( 0 );

  lastLesYes = new QRadioButton( this, "last lesson yes" );
  lastLesYes->setStyle( WindowsStyle );
  layOut.addWidget( lastLesYes, 3, 3 );

  lastLesNo = new QRadioButton( this, "last lesson no" );
  lastLesNo->setStyle( WindowsStyle );
  layOut.addWidget( lastLesNo, 3, 5 );

  lesGrp->insert( lastLesYes );
  lesGrp->insert( lastLesNo );
  lastLesNo->setChecked( true );

  // keep geometry stuff
  QLabel *keepGeoLabel = new QLabel( "Keep Geometry", this, "keep geom label" );
  keepGeoLabel->setMinimumSize( keepGeoLabel->sizeHint() );
  layOut.addWidget( keepGeoLabel, 4, 1 );

  QButtonGroup *geoGrp = new QButtonGroup( this, "geometry group" );
  geoGrp->setLineWidth( 0 );

  keepGeoYes = new QRadioButton( this, "keep geometry yes" );
  keepGeoYes->setStyle( WindowsStyle );
  layOut.addWidget( keepGeoYes, 4, 3 );

  keepGeoNo = new QRadioButton( this, "keep geometry no" );
  keepGeoNo->setStyle( WindowsStyle );
  layOut.addWidget( keepGeoNo, 4, 5 );

  geoGrp->insert( keepGeoYes );
  geoGrp->insert(keepGeoNo );
  keepGeoNo->setChecked( true );

  // keep splitter stuff
  QLabel *keepSplitLabel = new QLabel( "Keep Splitter Positions", this, "keep splitter label" );
  keepSplitLabel->setMinimumSize( keepSplitLabel->sizeHint() );
  layOut.addWidget( keepSplitLabel, 5, 1 );

  QButtonGroup *spltGrp = new QButtonGroup( this, "splitter group" );
  spltGrp->setLineWidth( 0 );

  keepSplitYes = new QRadioButton( this, "keep split yes" );
  keepSplitYes->setStyle( WindowsStyle );
  layOut.addWidget( keepSplitYes, 5, 3 );

  keepSplitNo = new QRadioButton( this, "keep split no" );
  keepSplitNo->setStyle( WindowsStyle );
  layOut.addWidget( keepSplitNo, 5, 5 );

  spltGrp->insert( keepSplitYes );
  spltGrp->insert( keepSplitNo );
  keepSplitNo->setChecked( true );

  // lesson directory stuff
  QLabel *lessonDirLabel = new QLabel( "Lesson Directory", this, "lesson dir label" );
  lessonDirLabel->setMinimumSize( lessonDirLabel->sizeHint() );
  layOut.addWidget( lessonDirLabel, 7, 1 );

  lessonDirEdit = new QLineEdit( this, "lesson dir edit" );
  layOut.addMultiCellWidget( lessonDirEdit, 7, 7, 2, 5 );


  layOut.activate();

}

/*!
  This is the destructor.  Currently it does nothing.
*/

SUConfigWidget::~SUConfigWidget(){}

/*!
  \fn QRadioButton *SUConfigWidget::getconYesPtr()
  This member returns a pointer to the autoconnect "Yes" button.
*/

/*!
  \fn QRadioButton *SUConfigWidget::getlesYesPtr()
  This member returns a pointer to the start on last lesson button.
*/

/*!
  \fn QRadioButton *SUConfigWidget::getgeoYesPtr()
  This member returns a pointer to the keep geometry button.
*/

/*!
  \fn QRadioButton *SUConfigWidget::getsplitYesPtr()
  This member returns a pointer to the keep splitter button.
*/

/*!
  \fn QLineEdit *SUConfigWidget::getDirEditPtr()
  This member returns a pointer to the lesson directory line edit.
*/
